@@ -13,16 +13,15 @@ api.createSubscriber('apns', 'token')
 
 """
 
-import urllib
-import urllib2
-import httplib
-import json
-import socket
+import requests
 
 class API(object):
 
-    def __init__(self, host='localhost', timeout=None):
-        self.host = host
+    def __init__(self, host='localhost', port=None, timeout=None):
+        if port:
+            self.host = host + ':' + str(port)
+        else:
+            self.host = host
         self.timeout = timeout
 
     def sendMessage(self, event, message, localized_messages={}, vars={}, data={}, sound=None):
@@ -46,7 +45,7 @@ class API(object):
             args['data.' + key] = value
         if sound:
             args['sound'] = sound
-        return self.request('event/' + event, post_args=args)
+        return self.request('event/' + event, post_args=args, method='POST')
 
     def createSubscriber(self, proto, token, lang='fr', badge=0):
         """Registers a device and returns an id for further communication"""
@@ -56,7 +55,7 @@ class API(object):
             "lang": lang,
             "badge": badge,
         }
-        return self.request('subscribers', post_args=data)
+        return self.request('subscribers', post_args=data, method='POST')
     
     def updateSubscriber(self, id, lang='fr', badge=0):
         """Ping to let pushd know the subscriber still exists
@@ -68,7 +67,7 @@ class API(object):
             "lang": lang,
             "badge": badge,
         }
-        return self.request('subscriber/' + id, post_args=data)
+        return self.request('subscriber/' + id, post_args=data, method='POST')
         
     def getSubscriber(self, id):
         """Get informations about a subscriber."""
@@ -76,10 +75,7 @@ class API(object):
 
     def deleteSubscriber(self, id):
         """Deletes a device using pushd generated id"""
-        conn = httplib.HTTPConnection(self.host)
-        conn.request('DELETE', '/subscriber/' + id)
-        resp = conn.getresponse()
-        return resp.read()
+        return self.request('subscriber/' + id, method='DELETE')
 
     def subscribe(self, id, event, ignore_message=False):
         """Subscribes to an event."""
@@ -87,18 +83,16 @@ class API(object):
             post_args = {"ignore_message": 1}
         else:
             post_args = {}
-        return self.request('subscriber/' + id + '/subscriptions/' + event, post_args=post_args)
+        return self.request('subscriber/' + id + '/subscriptions/' + event, post_args=post_args, method='POST')
 
     def bulkSubscribe(self, id, events):
         """Subscribe to multiple events. Taking dictionary."""
-        return self.request('subscriber/' + id + '/subscriptions', post_args=events)
+        headers = {'Content-type': 'application/json'}
+        return self.request('subscriber/' + id + '/subscriptions', post_args=events, headers=headers, method='POST')
 
     def unsubscribe(self, id, event):
         """Unsubscribe from an event."""
-        conn = httplib.HTTPConnection(self.host)
-        conn.request('DELETE', '/subscriber/' + id + '/subscriptions/' + event)
-        resp = conn.getresponse()
-        return resp.read()
+        return self.request('subscriber/' + id + '/subscriptions/' + event, method='DELETE')
 
     def getSubscriptions(self, id):
         """Fetchs subscriptions of a subscriber"""
@@ -108,30 +102,33 @@ class API(object):
         """Fetch only one subscription of a subscriber to check existence"""
         return self.request('subscriber/' + id + '/subscriptions/' + event)
 
-    def request(self, path, args=None, post_args=None):
+    def request(self, path, args=None, post_args=None, files=None, headers=None, method=None):
         args = args or {}
-        post_data = None if post_args is None else urllib.urlencode(post_args)
         try:
-            data = urllib2.urlopen("http://" + self.host + "/" + path + "?" +
-                                   urllib.urlencode(args),
-                                   post_data, timeout=self.timeout)
-            if data.code in [201, 204]:
+            r = requests.request(method or "GET",
+                                 "http://" + self.host + "/" + path, 
+                                 timeout=self.timeout,
+                                 params=args,
+                                 data=post_args,
+                                 files=files,
+                                 headers=headers)
+            if r.status_code in [201, 204]:
                 response = {
                     "result": "ok",
                 }
+            elif 'json' in r.headers['content-type']:
+                response = r.json()
             else:
-                response = json.loads(data.read())
-        except urllib2.HTTPError, err:
+                response = {
+                    "error": {
+                        "code": 404,
+                    }
+                }
+        except requests.HTTPError, e:
             response = {
                 "error": {
-                    "code": err.code,
+                    "code": e.status_code,
                 }
             }
-        except TypeError:
-            if self.timeout:
-                socket.setdefaulttimeout(self.timeout)
-            data = urllib2.urlopen("http://" + self.host + "/" + path + "?" +
-                                   urllib.urlencode(args), post_data)
-            response = json.loads(data.read())
 
         return response
